import os
import argparse
import numpy as np
from eval import evaluatemodel
parser = argparse.ArgumentParser(description='Experiments for optimizer')
parser.add_argument('-g','--gpu', default='0', type=str,
                help='gpu id to use from 0 to 3')
parser.add_argument('-w','--write', default=True, type=bool,
                help='writing the validation and test data')
parser.add_argument('-t','--train', default=True, type=bool,
                help='performing training or loading the model')
args = parser.parse_args()
os.environ["CUDA_VISIBLE_DEVICES"]=args.gpu

from reader import readfiles
from preprocess import split_data, augment_data
from train import loadmodel
from keras.callbacks import EarlyStopping, ModelCheckpoint, TensorBoard
from keras.models import model_from_json

def main():
    datapath = '/data/'

    print('************** Reading files *****************')
    data = readfiles(datapath)

    print('************** Splitting data *****************')
    train, val, test = split_data(data)

    print('************** Augmenting training data *****************')
    if args.train:
            train_aug = augment_data(train)

    resultspath = '/results/FinalPredictions'
    if not os.path.exists(resultspath):
        os.makedirs(resultspath)

    if args.train:
        print('************** Loading and setting up model *****************')
        feamodel = loadmodel(batch_size = 32)
        json_string = feamodel.to_json()
        with open((os.path.join(resultspath,'/results/model.json')), 'w') as f:
            f.write(json_string)

        early_stopping = EarlyStopping(monitor='val_loss', patience=30)
        model_checkpoint = ModelCheckpoint(os.path.join(resultspath,'/results/weights.h5'),
                                 monitor='val_loss', verbose=1, save_best_only=True)
        print('************** Training data *****************')
        feamodel.fit(x=train_aug[0][:-2], y=train_aug[1],  epochs=10,
                batch_size=512, validation_data=(val[0][:-2],val[1]),
                callbacks=[model_checkpoint,early_stopping], verbose = 1)

        # Loading the best weights
        feamodel.load_weights(os.path.join(resultspath,'/results/weights.h5'))
    else:
        print('************** Loading and setting up model *****************')
        with open((os.path.join(resultspath,'/results/model.json')), 'r') as f:
            loaded_model_json = f.read()
        feamodel = model_from_json(loaded_model_json)
        # Loading the best weights
        feamodel.load_weights(os.path.join(resultspath,'/results/weights.h5'))
        # compile the model to perform inference
        feamodel.compile(loss='mse', optimizer='adam')
        feamodel.summary()

    evaluatemodel(feamodel,val,test, train,resultspath,args.write)

if __name__ == '__main__':
    main()
