import numpy as np
import os
np.random.seed(1)

def augment_data(train_data):
    inputs, outputs = train_data
    naugments = 10

    aug_input_data = []
    for i, data in enumerate(inputs):
        if i < 3:
            aug_input_data.append(data.copy())
        else:
            aug_input_data.append(data.copy())

    for aug in range(naugments):
        shiftval = np.random.normal(loc = 0.0 , scale = 0.25)
        for i, data in enumerate(inputs):
            if i < 3:
                aug_input_data[i] = np.concatenate((aug_input_data[i],(data+shiftval)))
            else:
                aug_input_data[i] = np.concatenate((aug_input_data[i],data))

    aug_output_data = []
    for i, data in enumerate(outputs):
        if i < 3:
            aug_output_data.append(data.copy())
        else:
            aug_output_data.append(data.copy())

    for aug in range(naugments):
        for i, data in enumerate(outputs):
            if i < 3:
                aug_output_data[i] = np.concatenate((aug_output_data[i],data))
            else:
                aug_output_data[i] = np.concatenate((aug_output_data[i],data))

    return (aug_input_data, aug_output_data)

            


def split_data(data):
    inputs, outputs = data
    num_inputs = len(inputs)
    num_outputs = len(outputs)
    num_data_points = inputs[0].shape[0]

    indices = np.arange(num_data_points)
    np.random.shuffle(indices)

    train_split = 0.6
    val_split = 0.2
    test_split = 1 - train_split - val_split

    train_indices = indices[:int(num_data_points*train_split)]
    val_indices = indices[int(num_data_points*train_split):int(num_data_points*(train_split+val_split))]
    test_indices = indices[int(num_data_points*(train_split+val_split)):]

    inputs = list(inputs)
    train_inputs = []
    val_inputs = []
    test_inputs = []

    for data in inputs:
        train_inputs.append(data[train_indices])
        val_inputs.append(data[val_indices])
        test_inputs.append(data[test_indices])

    outputs = list(outputs)
    train_outputs = []
    val_outputs = []
    test_outputs = []

    for data in outputs:
        train_outputs.append(data[train_indices])
        val_outputs.append(data[val_indices])
        test_outputs.append(data[test_indices])

    train = (train_inputs, train_outputs)
    test = (test_inputs, test_outputs)
    val = (val_inputs, val_outputs)

    return train, val, test



