import numpy as np
import os
import csv
import pickle

def readmeshfile(filename):
    with open(filename,'r') as f:
        lines = f.readlines()
        pts = [(points.strip().split())[:3] for points in lines]
    u_ctrl_pts = 17
    v_ctrl_pts = 12
    mesh = np.zeros((u_ctrl_pts,v_ctrl_pts,3),dtype='float32')
    for u in range(u_ctrl_pts):
        for v in range(v_ctrl_pts):
            mesh[u,v,:] = np.array(pts[5 + v_ctrl_pts*u + v]).astype('float32')
    return mesh

def readoutputfile(filename):
    with open(filename,'r') as f:
        lines = f.readlines()
        pts = [(points.strip().split())[:3] for points in lines]
    u_ctrl_pts = 17
    v_ctrl_pts = 12
    displacements = np.zeros((u_ctrl_pts,v_ctrl_pts,3),dtype='float32')
    for u in range(u_ctrl_pts):
        for v in range(v_ctrl_pts):
            displacements[u,v,:] = np.array(pts[1 + v_ctrl_pts*u + v]).astype('float32')
    return displacements

def get_coaptation_val(filename):
    with open(filename,'r') as f:
        lines = f.readlines()
    last_line = lines[-1]
    last_line = last_line.strip().split()
    coaptation_val = float(last_line[0])
    return coaptation_val



def _readfiles(folderpath):

    with open(os.path.join(folderpath,"success_data_list.csv"),'r') as f:
        data_list = f.readlines()

    data_size = len(data_list)
    geometry_index = 0
    pressure_index = 1
    thickness_fn_index = 2
    thickness_index = 3

    x1_data, x2_data, x3_data, x4_data, x5_data = [], [], [], [], []
    x6_data, x7_data = [], []
    y1_data, y2_data, y3_data, y4_data = [], [], [], []
    success_list = []
    failure_list = []
    it = 0
    for row in data_list:
        row = row.strip().split(',')
        y1_datapoint = y2_datapoint = y3_datapoint = x1_datapoint = x2_datapoint = x3_datapoint= None
        geom_num = row[geometry_index]
        pressure = row[pressure_index]
        thickness = row[thickness_index]
        thickness_fn = row[thickness_fn_index]
        try:
            y1_datapoint = readoutputfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','sh.rest.{}.0{}'.format(140, 1)))
            y2_datapoint = readoutputfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','sh.rest.{}.0{}'.format(140, 2)))
            y3_datapoint = readoutputfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','sh.rest.{}.0{}'.format(140, 3)))
            y4_datapoint = get_coaptation_val(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','coaptation'))
            if y1_datapoint is not None and y2_datapoint is not None and y3_datapoint is not None :
                x1_datapoint = readmeshfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','smesh.{}.dat'.format(1)))
                x2_datapoint = readmeshfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','smesh.{}.dat'.format(2)))
                x3_datapoint = readmeshfile(os.path.join(folderpath,'runs','{}_{}_{}'.format(geom_num,thickness_fn,pressure),'setup','smesh.{}.dat'.format(3)))
            if x1_datapoint is not None and x2_datapoint is not None and x3_datapoint is not None :
                it += 1
                x1_data.append(x1_datapoint); x2_data.append(x2_datapoint); x3_data.append(x3_datapoint);
                x4_data.append(float(pressure)); x5_data.append(float(thickness)); x6_data.append(geom_num); x7_data.append(thickness_fn) 
                y1_data.append(y1_datapoint); y2_data.append(y2_datapoint); y3_data.append(y3_datapoint); y4_data.append(y4_datapoint)
                success_list.append(row)
        except FileNotFoundError:
            print(it)
            failure_list.append(row)
            pass

    # with open(os.path.join(folderpath,"success_data_list.csv"),'w') as f:
    #     writer = csv.writer(f, lineterminator='\n')
    #     writer.writerows(success_list)

    # with open(os.path.join(folderpath,"failure_data_list.csv"),'w') as f:
    #     writer = csv.writer(f, lineterminator='\n')
    #     writer.writerows(failure_list)

    inputs = (np.array(x1_data), np.array(x2_data), np.array(x3_data),
             np.array(x4_data), np.array(x5_data), np.array(x6_data),
             np.array(x7_data))
    outputs = (np.array(y1_data), np.array(y2_data), np.array(y3_data),
             np.array(y4_data))

    with open(os.path.join(folderpath,'input_data.pkl'), 'wb') as handle:
        pickle.dump(inputs, handle, protocol=pickle.HIGHEST_PROTOCOL)

    with open(os.path.join(folderpath,'output_data.pkl'), 'wb') as handle:
        pickle.dump(outputs, handle, protocol=pickle.HIGHEST_PROTOCOL)

    return inputs, outputs


def readfiles(folderpath):

    try:
        inputs = pickle.load(open(os.path.join(folderpath,'input_data.pkl'), "rb"))
        outputs = pickle.load(open(os.path.join(folderpath,'output_data.pkl'), "rb"))
    except FileNotFoundError:
        print('pickle files not found, loading raw files and generating them')
        inputs, outputs = _readfiles(folderpath)
    inputs = list(inputs)
    outputs = list(outputs)
    inputs = [np.expand_dims(x, 1) if x is not None and x.ndim == 1 else x for x in inputs]
    return inputs, outputs








