import os
os.environ['TF_CPP_MIN_LOG_LEVEL']='3'
os.environ['KERAS_BACKEND']="tensorflow"
os.environ['CUDA_VISIBLE_DEVICES']='0'
import tensorflow as tf
config = tf.ConfigProto()
config.gpu_options.allow_growth = True
sess = tf.Session(config=config)
from keras import backend as K
K.set_session(sess)
K.set_image_data_format('channels_last')
from keras import backend as K
from keras.models import Model, Sequential
from keras import layers, regularizers
from keras.layers.core import Activation, Flatten, Dense, Dropout, RepeatVector
from keras.layers import Conv2D, SeparableConv2D, Conv2DTranspose, concatenate, Add
from keras.layers.pooling import MaxPooling2D
from keras.layers import Input, Reshape
from keras.layers.normalization import BatchNormalization
from keras.regularizers import l2
import numpy as np
from keras.models import load_model
from preprocess import augment_data
# from keras.losses import logcosh


def loadmodel(batch_size = 32):
    input_shape = (17,12,3)
    x1 = Input(shape = input_shape, name = 'leaflet1')
    x2 = Input(shape = input_shape, name = 'leaflet2')
    x3 = Input(shape = input_shape, name = 'leaflet3')
    x4 = Input(shape = (1,), name = 'pressure')
    x5 = Input(shape = (1,), name = 'thickness')

    encoder = Sequential()
    encoder.add(Conv2D(3, (5, 5), kernel_regularizer = regularizers.l2(10e-6),
                activity_regularizer = regularizers.l1(10e-6)))
    encoder.add(BatchNormalization(axis=-1))
    encoder.add(Activation('relu'))
    # encoder.add(MaxPooling2D((2, 2)))
    encoder.add(Conv2D(8, (3, 3)))
    encoder.add(BatchNormalization(axis=-1))
    encoder.add(Activation('relu'))
    encoder.add(Dropout(0.25))
    # encoder.add(MaxPooling2D((2, 2)))
    encoder.add(Conv2D(16, (3, 3)))
    encoder.add(BatchNormalization(axis=-1))
    encoder.add(Activation('relu'))
    encoder.add(Dropout(0.25))
    # encoder.add(MaxPooling2D((2, 2)))
    encoder.add(Conv2D(32, (3, 3)))
    encoder.add(BatchNormalization(axis=-1))
    encoder.add(Activation('relu'))
    encoder.add(Dropout(0.25))
    # encoder.add(regularizers.activity_l1(10e-5))

    # encoder.add(MaxPooling2D((2, 2)))
    encoder.add(Flatten())

    branch1 = encoder(x1)
    branch2 = encoder(x2)
    branch3 = encoder(x3)

    repeated_x4 = RepeatVector(20)(x4)
    repeated_x5 = RepeatVector(20)(x5)



    # FC1 = concatenate([branch1, branch2, branch3, K.repeat_elements(x4,10,-1), K.repeat_elements(x5,10,-1)])
    FC1 = concatenate([branch1, branch2, branch3, Flatten()(repeated_x4), Flatten()(repeated_x5)])

    FC2 = Dense(64, activation='relu')(FC1)
    FC2 = Dropout(0.2)(FC2)
    FC3 = Dense(48)(FC2)
    FC3 = BatchNormalization(axis=-1)(FC3)
    FC3 = Activation('relu')(FC3)
    FC2 = Dropout(0.2)(FC2)
    code_layer = Dense(48, activation='relu', name='code_layer', kernel_regularizer = regularizers.l2(10e-6),
                activity_regularizer = regularizers.l1(10e-6))(FC3)
    coaptation_layer = Dense(20, activation='relu')(code_layer)
    coaptation_layer = Dropout(0.2)(coaptation_layer)
    coaptation_layer = Dense(10, activation='relu')(coaptation_layer)
    coaptation_layer = Dropout(0.1)(coaptation_layer)
    coaptation_layer = Dense(1, activation='relu')(coaptation_layer)

    dense_layer1 = Dense(64, activation='relu')(code_layer)
    dense_layer2 = Dense(64, activation='relu')(code_layer)
    dense_layer3 = Dense(64, activation='relu')(code_layer)


    dense_layer1 = Dense(8 * 9 * 4, activation='relu')(dense_layer1)
    dense_layer2 = Dense(8 * 9 * 4, activation='relu')(dense_layer2)
    dense_layer3 = Dense(8 * 9 * 4, activation='relu')(dense_layer3)


    if K.image_data_format() == 'channels_first':
        output_shape = (batch_size, 8, 9, 4)
    else:
        output_shape = (batch_size, 9, 4, 8)


    disp1 = Reshape(output_shape[1:])(dense_layer1)
    disp2 = Reshape(output_shape[1:])(dense_layer2)
    disp3 = Reshape(output_shape[1:])(dense_layer3)

    # deconv_layer1 = Conv2DTranspose(16, (5, 5), strides=(1, 1))
    # deconv_layer2 = Conv2DTranspose(3, (3, 3), strides=(1, 1))
    # deconv_layer3 = Conv2DTranspose(3, (3, 3), strides=(1, 1))

    decoder = Sequential()
    decoder.add(Conv2DTranspose(8, (5, 5), strides=(1, 1)))
    decoder.add(Dropout(0.1))
    decoder.add(Conv2DTranspose(6, (3, 3), strides=(1, 1)))
    decoder.add(Dropout(0.1))
    decoder.add(Conv2DTranspose(4, (3, 3), strides=(1, 1)))
    decoder.add(Conv2DTranspose(3, (1, 1), strides=(1, 1), activation='linear'))

    disp1 = decoder(disp1)
    disp2 = decoder(disp2)
    disp3 = decoder(disp3)


    K.set_epsilon(1e-09)
    def disp_loss(y_true, y_pred, clip_delta=1.0):
        se1 = K.sum(((1.0+y_true)/K.max(1.0 + y_true))*K.square(y_pred - y_true), axis=-1)
        # ae = K.mean(K.abs(y_true/K.max(y_true))*K.max(y_pred - y_true), axis=-1)
        se2 = K.mean(K.square(y_pred - y_true), axis=-1)
        return se1 + se2


    model=Model(inputs=[x1,x2,x3,x4,x5], outputs=[disp1, disp2, disp3, coaptation_layer])

    model.compile(loss='mse', optimizer='adam')
    model.summary()
    return model
